/*
 * $Header$
 * $Revision: 1235 $
 * $Date: 2006-11-08 18:02:21 +0100 (Wed, 08 Nov 2006) $
 *
 * ====================================================================
 *
 * Copyright 2000-2002 bob mcwhirter & James Strachan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *   * Neither the name of the Jaxen Project nor the names of its
 *     contributors may be used to endorse or promote products derived 
 *     from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ====================================================================
 * This software consists of voluntary contributions made by many 
 * individuals on behalf of the Jaxen Project and was originally 
 * created by bob mcwhirter <bob@werken.com> and 
 * James Strachan <jstrachan@apache.org>.  For more information on the 
 * Jaxen Project, please see <http://www.jaxen.org/>.
 * 
 * $Id: LocationPath.java 1235 2006-11-08 17:02:21Z elharo $
 */

package org.jaxen.expr;

import java.util.List;

/**
 * Represents an XPath location path such as <code>//foo/bar</code>
 * or <code>pre:baz[position()=last()]</code>.
 * This is production 1 in the 
 * <a href="http://www.w3.org/TR/xpath#NT-LocationPath">XPath 1.0 specification</a>:
 * 
 * <pre>[1]  LocationPath ::= RelativeLocationPath    
 *                    | AbsoluteLocationPath</pre>
 * 
 */
public interface LocationPath extends Expr
{
    
    /**
     * Add the next step to this location path.
     * 
     * @param step
     */
    void addStep(Step step);

    /**
     * Returns the ordered list of steps in this location path.
     * This list may be live.
     * 
     * @return the ordered list of steps in this location path
     */
    List getSteps();
    
    /**
     * Returns true if this is an absolute location path; false if it isn't. 
     * Absolute location paths all begiune with <code>/</code> 
     * or <code>//</code>.
     * 
     * @return true if this is an absol;ute location path; false if it isn't
     */
    boolean isAbsolute();
}
